@@ -58,7 +58,7 @@ static prelude_string_t *get_message_ident(prelude_ident_t *ident)
         if ( ret < 0 )
                 return NULL;
 
-        ret = prelude_string_sprintf(str, "%" PRELUDE_PRIu64, prelude_ident_inc(ident));
+        ret = prelude_ident_generate(ident, str);
         if ( ret < 0 ) {
                 prelude_string_destroy(str);
                 return NULL;
@@ -127,8 +127,6 @@ static int find_absolute_path(const char *cwd, const char *file, char **path)
  * request fails. If size was equal to 0, either NULL or a pointer suitable to be
  * passed to free() is returned.  If  realloc() fails, the original block is left
  * untouched - it is not freed nor moved.
- *
- * Returns: a pointer to the newly allocated memory.
  */
 void *_prelude_realloc(void *ptr, size_t size)
 {
@@ -493,8 +491,6 @@ prelude_msg_priority_t _idmef_impact_severity_to_msg_priority(idmef_impact_sever
 
 int _idmef_message_assign_missing(prelude_client_t *client, idmef_message_t *msg)
 {
-        int ret;
-        idmef_time_t *time;
         idmef_alert_t *alert;
         idmef_heartbeat_t *heartbeat;
         prelude_ident_t *ident = prelude_client_get_unique_ident(client);
@@ -504,30 +500,11 @@ int _idmef_message_assign_missing(prelude_client_t *client, idmef_message_t *msg
 
                 if ( ! idmef_alert_get_messageid(alert) )
                         idmef_alert_set_messageid(alert, get_message_ident(ident));
-
-                time = idmef_alert_get_create_time(alert);
-                if ( idmef_time_get_sec(time) == 0 ) {
-                        ret = idmef_time_new_from_gettimeofday(&time);
-                        if ( ret < 0 )
-                                return ret;
-
-                        idmef_alert_set_create_time(alert, time);
-                }
-
         } else {
                 heartbeat = idmef_message_get_heartbeat(msg);
 
                 if ( ! idmef_heartbeat_get_messageid(heartbeat) )
                         idmef_heartbeat_set_messageid(heartbeat, get_message_ident(ident));
-
-                time = idmef_heartbeat_get_create_time(heartbeat);
-                if ( idmef_time_get_sec(time) == 0 ) {
-                        ret = idmef_time_new_from_gettimeofday(&time);
-                        if ( ret < 0 )
-                                return ret;
-
-                        idmef_heartbeat_set_create_time(heartbeat, time);
-                }
         }
 
         return 0;
