@@ -206,15 +206,13 @@ static const char *client_get_status(prelude_client_t *client)
 
 
 
-
-static void heartbeat_expire_cb(void *data)
+static void gen_heartbeat(prelude_client_t *client)
 {
         int ret;
         idmef_time_t *time;
         prelude_string_t *str;
         idmef_message_t *message;
         idmef_heartbeat_t *heartbeat;
-        prelude_client_t *client = data;
 
         ret = idmef_message_new(&message);
         if ( ret < 0 ) {
@@ -260,14 +258,20 @@ static void heartbeat_expire_cb(void *data)
 
   out:
         idmef_message_destroy(message);
+}
+
+
+static void heartbeat_expire_cb(void *data)
+{
+        prelude_client_t *client = data;
+
+        gen_heartbeat(client);
 
         if ( client->status != CLIENT_STATUS_EXITING )
                 prelude_timer_reset(&client->heartbeat_timer);
 }
 
 
-
-
 static void setup_heartbeat_timer(prelude_client_t *client, int expire)
 {
         prelude_timer_set_data(&client->heartbeat_timer, client);
@@ -830,10 +834,8 @@ static int set_profile(prelude_option_t *opt, const char *optarg, prelude_string
 
         prelude_client_profile_get_config_filename(client->profile, buf, sizeof(buf));
 
-        if ( client->config_filename )
-                free(client->config_filename);
-
-        client->config_filename = strdup(buf);
+        prelude_client_set_config_filename(client, buf);
+        client->config_external = FALSE;
 
         return 0;
 }
@@ -1038,7 +1040,7 @@ int prelude_client_new(prelude_client_t **client, const char *profile)
         prelude_thread_mutex_init(&new->msgbuf_lock, NULL);
         prelude_timer_init_list(&new->heartbeat_timer);
 
-        new->flags = PRELUDE_CLIENT_FLAGS_HEARTBEAT|PRELUDE_CLIENT_FLAGS_CONNECT;
+        new->flags = PRELUDE_CLIENT_FLAGS_HEARTBEAT|PRELUDE_CLIENT_FLAGS_CONNECT|PRELUDE_CLIENT_FLAGS_AUTOCONFIG;
         new->permission = PRELUDE_CONNECTION_PERMISSION_IDMEF_WRITE;
 
         ret = idmef_analyzer_new(&new->analyzer);
@@ -1183,16 +1185,21 @@ int prelude_client_start(prelude_client_t *client)
 
 
         if ( (client->cpool || client->heartbeat_cb) && client->flags & PRELUDE_CLIENT_FLAGS_HEARTBEAT ) {
+
+                client->status = CLIENT_STATUS_STARTING;
+                client->_analyzer_copy = client->analyzer;
+                gen_heartbeat(client);
+
+                /*
+                 * We use a copy of the analyzer object from the timer,
+                 * since it might be run asynchronously.
+                 */
                 ret = idmef_analyzer_clone(client->analyzer, &client->_analyzer_copy);
                 if ( ret < 0 )
                         return ret;
 
-                client->status = CLIENT_STATUS_STARTING;
-                /*
-                 * this will reset, and thus initialize the timer.
-                 */
-                heartbeat_expire_cb(client);
                 client->status = CLIENT_STATUS_RUNNING;
+                prelude_timer_init(&client->heartbeat_timer);
         }
 
         return 0;
@@ -1304,6 +1311,9 @@ void prelude_client_set_connection_pool(prelude_client_t *client, prelude_connec
         prelude_return_if_fail(client);
         prelude_return_if_fail(pool);
 
+        if ( client->cpool )
+                prelude_connection_pool_destroy(client->cpool);
+
         client->cpool = pool;
 }
 
@@ -1349,7 +1359,10 @@ void prelude_client_destroy(prelude_client_t *client, prelude_client_exit_status
 
         prelude_timer_destroy(&client->heartbeat_timer);
 
-        if ( status == PRELUDE_CLIENT_EXIT_STATUS_SUCCESS && client->flags & PRELUDE_CLIENT_FLAGS_HEARTBEAT ) {
+        if ( client->status >= CLIENT_STATUS_STARTING     &&
+             status == PRELUDE_CLIENT_EXIT_STATUS_SUCCESS &&
+             client->flags & PRELUDE_CLIENT_FLAGS_HEARTBEAT ) {
+
                 client->status = CLIENT_STATUS_EXITING;
                 heartbeat_expire_cb(client);
         }
@@ -1385,15 +1398,18 @@ int prelude_client_set_flags(prelude_client_t *client, prelude_client_flags_t fl
         client->flags = flags;
 
         if ( flags & PRELUDE_CLIENT_FLAGS_ASYNC_TIMER ) {
-                ret = prelude_async_init();
                 prelude_async_set_flags(PRELUDE_ASYNC_FLAGS_TIMER);
+                ret = prelude_async_init();
         }
 
         if ( flags & PRELUDE_CLIENT_FLAGS_ASYNC_SEND ) {
-                ret = prelude_async_init();
                 prelude_msgbuf_set_flags(client->msgbuf, PRELUDE_MSGBUF_FLAGS_ASYNC);
+                ret = prelude_async_init();
         }
 
+        if ( ! (flags & PRELUDE_CLIENT_FLAGS_AUTOCONFIG) )
+                prelude_client_set_config_filename(client, NULL);
+
         return ret;
 }
 
@@ -1487,14 +1503,20 @@ const char *prelude_client_get_config_filename(prelude_client_t *client)
 int prelude_client_set_config_filename(prelude_client_t *client, const char *filename)
 {
         prelude_return_val_if_fail(client, prelude_error(PRELUDE_ERROR_ASSERTION));
-        prelude_return_val_if_fail(filename, prelude_error(PRELUDE_ERROR_ASSERTION));
 
-        if ( client->config_filename )
+        if ( client->config_filename ) {
                 free(client->config_filename);
+                client->config_filename = NULL;
+        }
 
-        client->config_filename = strdup(filename);
-        if ( ! client->config_filename )
-                return prelude_error_from_errno(errno);
+        if ( ! filename )
+                client->flags &= ~PRELUDE_CLIENT_FLAGS_AUTOCONFIG;
+
+        else {
+                client->config_filename = strdup(filename);
+                if ( ! client->config_filename )
+                        return prelude_error_from_errno(errno);
+        }
 
         client->config_external = TRUE;
 
